<?php
/**
 * Kunena Forum SEF extension for Joomla!
 *
 * @author      $Author: David Jozefov $
 * @modified    $Author: Matias Aguirre $
 * @copyright   ARTIO s.r.o., http://www.artio.cz
 * @package     JoomSEF
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access.');

class SefExt_com_kunena extends SefExt
{

    function getNonSefVars(&$uri)
    {
        $this->_createNonSefVars($uri);
        
        return array($this->nonSefVars, $this->ignoreVars);
    }
    
    function _createNonSefVars(&$uri)
    {
        if (isset($this->nonSefVars) && isset($this->ignoreVars))
            return;
            
        $this->params = SEFTools::getExtParams('com_kunena');
        
        $this->nonSefVars = array();
        $this->ignoreVars = array();
        
        // Set non-sef vars according to settings
        $doOptimize = in_array($uri->getVar('do'), array('reply', 'quote', 'delete', 'edit', 'move', 'sticky', 'lock'))
        || ($uri->getVar('func') == 'report');
        if ($this->params->get('idExclude', true) && $doOptimize) {
            if (!is_null($uri->getVar('id')))
                $this->nonSefVars['id'] = $uri->getVar('id');
            if (!is_null($uri->getVar('replyto')))
                $this->nonSefVars['replyto'] = $uri->getVar('replyto');
            if (!is_null($uri->getVar('msg_id')))
                $this->nonSefVars['msg_id'] = $uri->getVar('msg_id');
        }
        if ($this->params->get('optimize', true) && $doOptimize) {
            if (!is_null($uri->getVar('fb_thread')))
                $this->nonSefVars['fb_thread'] = $uri->getVar('fb_thread');
            if (!is_null($uri->getVar('name')))
                $this->nonSefVars['name'] = $uri->getVar('name');
        }
        if ($this->params->get('doExclude', true)) {
            if (!is_null($uri->getVar('do')))
                $this->nonSefVars['do'] = $uri->getVar('do');
        }

        $func = $uri->getVar('func');
        if (is_null($func)) {
            $func = $uri->getVar('task');
        }
        if (in_array($func, array('search', 'advsearch'))) {
            if (!is_null($uri->getVar('limitstart')))
                $this->ignoreVars['limitstart'] = $uri->getVar('limitstart');
            if (!is_null($uri->getVar('limit')))
                $this->ignoreVars['limit'] = $uri->getVar('limit');
        }
        
        if (!is_null($uri->getVar('q')))
            $this->ignoreVars['q'] = $uri->getVar('q');
        if (!is_null($uri->getVar('beforeafter')))
            $this->ignoreVars['beforeafter'] = $uri->getVar('beforeafter');
        if (!is_null($uri->getVar('catids')))
            $this->ignoreVars['catids'] = $uri->getVar('catids');
        if (!is_null($uri->getVar('exactname')))
            $this->ignoreVars['exactname'] = $uri->getVar('exactname');
        if (!is_null($uri->getVar('order')))
            $this->ignoreVars['order'] = $uri->getVar('order');
        if (!is_null($uri->getVar('searchdate')))
            $this->ignoreVars['searchdate'] = $uri->getVar('searchdate');
        if (!is_null($uri->getVar('searchuser')))
            $this->ignoreVars['searchuser'] = $uri->getVar('searchuser');
        if (!is_null($uri->getVar('sortby')))
            $this->ignoreVars['sortby'] = $uri->getVar('sortby');
        if (!is_null($uri->getVar('titleonly')))
            $this->ignoreVars['titleonly'] = $uri->getVar('titleonly');
        if (!is_null($uri->getVar('childforums')))
            $this->ignoreVars['childforums'] = $uri->getVar('childforums');
        if (!is_null($uri->getVar('direction')))
            $this->ignoreVars['direction'] = $uri->getVar('direction');
        if (!is_null($uri->getVar('orderby')))
            $this->ignoreVars['orderby'] = $uri->getVar('orderby');
    }

    function beforeCreate(&$uri)
    {
        global $fbConfig, $mainframe;
        
        if (!isset($fbConfig->messages_per_page)) {
            // Load configuration
            $file = JPATH_ROOT.DS.'components'.DS.'com_kunena'.DS.'lib'.DS.'kunena.config.class.php';
            if (!file_exists($file)) {
                return;
            }
            
            require_once($file);
            
            // Create configuration object
            $fbConfig = new CKunenaConfig();
            $fbConfig->load();
        }
                
        $database =& JFactory::getDBO();
        $params = SEFTools::getExtParams('com_kunena');
        
        $vars = $uri->getQuery(true);
        extract($vars);
        
        if (isset($func) && ($func == 'showcat') && isset($id)) {
            $uri->delVar('id');
        }
        elseif( isset($id) && $params->get('smarturls', true) && (@$func != 'post' && @$do != 'edit')) {
            // Find the root msg
            $oldid = $id;
            
            $query = "SELECT `id` FROM `#__kunena_messages` WHERE (`parent` = '0') AND (`thread` = (SELECT `thread` FROM `#__kunena_messages` WHERE `id` = '$id'))";
            $database->setQuery($query);
            $id = $database->loadResult();
            
            $uri->setVar('id', $id);
            
            if( $oldid != $id ) {
                // Get the message number
                $query = "SELECT COUNT(`id`) FROM `#__kunena_messages` WHERE (`id` < '$oldid') AND (`thread` = (SELECT `thread` FROM `#__kunena_messages` WHERE `id` = '$oldid')) ORDER BY `id`";
                $database->setQuery($query);
                $num = $database->loadResult();
                
                // Compute the limitstart
                $l = $fbConfig->messages_per_page;
                if( $num >= $l ) {
                    $ls = intval($num / $l) * $l;
                    $uri->setVar('limitstart', $ls);
                }
            }
        }
        
        if( is_null($uri->getVar('limitstart')) && !is_null($uri->getVar('limit')) ) {
            $uri->delVar('limit');
        }
        
        if( !is_null($uri->getVar('limitstart')) && is_null($uri->getVar('limit')) ) {
            if( !in_array($uri->getVar('func'), array('search', 'advsearch')) ) {
                $uri->setVar('limit', $fbConfig->messages_per_page);
            }
        }
        
        // Category listing
        if( ($uri->getVar('func') == 'showcat') && ($uri->getVar('catid') == '0') ) {
            $uri->delVar('catid');
            $uri->setVar('func', 'listcat');
        }
        
        // Go variable
        if( !is_null($uri->getVar('Go')) ) {
            $uri->delVar('Go');
        }
    }
    
    function GetCategories($id)
    {
        $db =& JFactory::getDBO();
        $sefConfig =& SEFConfig::getConfig();
        
        $categories = array();
        
        // Check if we want our URLs translated
        if ($sefConfig->translateNames) {
            $jfTranslate = ', `id`';
        } else {
            $jfTranslate = '';
        }
        
        while ($id > 0) {
            $db->setQuery("SELECT `name`, `parent`$jfTranslate FROM `#__kunena_categories` WHERE `id` = '{$id}'");
            $row = $db->loadObject();
            
            if (is_null($row)) {
                break;
            }
            
            $row->id = $id;
            $name = $this->BuildCategoryName($row);
            array_unshift($categories, $name);
            
            $id = $row->parent;
        }
        
        return $categories;
    }
    
    function BuildCategoryName($category)
    {
        $name = array();
        $this->AddCategoryNamePart($name, $category, $this->params->get('categoryname1', 'none'));
        $this->AddCategoryNamePart($name, $category, $this->params->get('categoryname2', 'title'));
        
        return implode('-', $name);
    }
    
    function AddCategoryNamePart(&$name, $category, $part)
    {
        switch ($part)
        {
            case 'id':
                $name[] = $category->id;
                return;
            case 'title':
                $name[] = $category->name;
                return;
            case 'text':
                $name[] = 'forum';
                return;
        }
    }
    
    function GetMessageTitle($id)
    {
        $db =& JFactory::getDBO();
        $sefConfig =& SEFConfig::getConfig();
        
        // Check if we want our URLs translated
        if ($sefConfig->translateNames) {
            $jfTranslate = ', `id`';
        } else {
            $jfTranslate = '';
        }
        
        $db->setQuery("SELECT `subject`$jfTranslate FROM `#__kunena_messages` WHERE `id` = '{$id}'");
        $row = $db->loadObject();
        
        if (is_null($row)) {
            return '';
        }
        
        $row->id = $id;
        $name = $this->BuildMessageName($row);
        
        return $name;
    }

    function BuildMessageName($message)
    {
        $name = array();
        $this->AddMessageNamePart($name, $message, $this->params->get('topicname1', 'none'));
        $this->AddMessageNamePart($name, $message, $this->params->get('topicname2', 'title'));
        
        return implode('-', $name);
    }
    
    function AddMessageNamePart(&$name, $message, $part)
    {
        switch ($part)
        {
            case 'id':
                $name[] = $message->id;
                return;
            case 'title':
                $name[] = $message->subject;
                return;
            case 'text':
                $name[] = 'topic';
                return;
        }
    }
    
    function create(&$uri)
    {
        $vars = $uri->getQuery(true);
        extract($vars);

        // JF translate extension.
        $sefConfig =& SEFConfig::getConfig();
        $database =& JFactory::getDBO();

        $jfTranslate = $sefConfig->translateNames ? ', `id`' : '';

        // load params
        $this->params = SEFTools::getExtParams('com_kunena');

        $catRewrite = true;
        $msgRewrite = true;
        $usrRewrite = true;

        if ($msgRewrite || $catRewrite) {
            if ($catRewrite && !empty($catid)) {
                $categories = $this->GetCategories($catid);
            }
            if (isset($id)) $msgID = $id;
            elseif (isset($msg_id)) $msgID = $msg_id;
            elseif (isset($replyto)) $msgID = $replyto;
            else $msgID = null;
            if ($msgRewrite && !empty($msgID)) {
                $msgTitle = $this->GetMessageTitle($msgID);
            }
        }
        if ($usrRewrite && isset($userid)) {
            $query = "
        	SELECT `username`
        	FROM `#__users`
        	WHERE `id` = $userid
        	";
            $database->setQuery($query);
            $usrTitle = $database->loadResult();
        }

        // Set non-sef vars according to settings
        $this->_createNonSefVars($uri);

        // this needs to follow previous that use do
        if ($this->params->get('doExclude', true)) {
            unset($do);
        }

        // if task is not set, use do
        if (empty($task) && isset($func) && $func == 'post' && isset($do)) {
            $task = $do;
            unset($func); unset($do);
        }

        if (empty($task) && isset($func)) {
            $task = $func;
            unset($func);
        }

        // First subdir
        if (!empty($option)) {
            $title[] = JoomSEF::_getMenuTitle($option, @$task, @$Itemid);
        }

        // Category
        if (isset($categories) && !empty($categories)) {
            $addCat = $this->params->get('categories', '2');
            if ($addCat == '2') {
                $title = array_merge($title, $categories);
            }
            else if ($addCat == '1' || empty($msgID)) {
                $title[] = $categories[count($categories) - 1];
            }
        }

        // Topic
        if (isset($msgTitle) && !empty($msgTitle) && isset($task) && ($task != 'showcat')) {
            //$title[] = (!isset($do) && !isset($func)) ? $msgTitle.$sefConfig->suffix : $msgTitle;
            $title[] = $msgTitle;
        }

        if( isset($task) && in_array($task, array('search', 'advsearch')) ) {
            $title[] = JText::_('search');
            if (isset($limitstart)) {
                unset($limitstart);
            }
            if (isset($limit)) {
                unset($limit);
            }
            unset($task);
        }
        
        // Page number
        if( isset($limitstart) ) {
            // Limit should be set due to beforeCreate function, but to be sure
            if( !isset($limit) ) {
                global $fbConfig;
                $limit = $fbConfig->messages_per_page;
            }
            
            $pageNum = intval($limitstart / $limit) + 1;
            $pagetext = strval($pageNum);
            if (($cnfPageText = $sefConfig->getPageText())) {
                $pagetext = str_replace('%s', $pageNum, $cnfPageText);
            }
            $title[] = $pagetext;
        }

        // Selection
        if (@$task == 'latest' && !empty($sel)) {
            $task .= '-'.$sel;
        }

        // View
        if (isset($view)) {
            $title[] = (!isset($do) && !isset($func)) ? $view.$sefConfig->suffix : $view;
        }

        // User
        if (isset($usrTitle) && !empty($usrTitle)) {
            if (@$task == 'fbprofile') {
                $title[] = 'users';
                $task = null;
            }
            if( @$task == 'showprf' ) {
                $task = null;
            }
            $title[] = !isset($do) ? $usrTitle.$sefConfig->suffix : $usrTitle;
        }

        if (isset($page) && $page > 1) {
            $title[] = $page;
        }
        
        // Func and do
        if (isset($do) || isset($func)) {
            if (isset($func)) $oper[] = $func;
            if (isset($do))   $oper[] = $do;
            $title[] = join('-', $oper).$sefConfig->suffix;
        }

        if( in_array(@$task, array('showcat', 'view')) ) {
            unset($task);
        }
        
        // Fix some task texts
        if (isset($task)) {
            $task = str_replace(array('listcat', 'userlist'), array('categories', 'users'), $task);
        }

        $newUri = $uri;
        if (count($title) > 0) {
            $newUri = JoomSEF::_sefGetLocation($uri, $title, @$task, @$limit, @$limitstart, @$lang, $this->nonSefVars, $this->ignoreVars, null, null, true);
        }

        return $newUri;
    }

}
?>
